/* (c) Peter Mueller, 2014 - All rights reserved. Reproduction, modification,
   use or disclosure to third parties without express
   authority is forbidden.
 */

#include <msp430.h>
#include <stdint.h>

#include "timer.h"
#include "ringbuf.h"

typedef struct {
  TIMER_STATE_T timerStatus;
  bool cyclic;
  uint8_t preset;
  uint8_t elapsedTicksSinceStart;
  FIFO_T *rbuf;
  uint16_t evTimeout;
}TIMER_T;

#define MAX_TIMERS 3
#define NO_TIMERS_LEFT 127

static TIMER_T timers[MAX_TIMERS];

void timerInit(void){
	uint8_t n;
	for(n=0; n<sizeof(timers)/sizeof(TIMER_T);n++){
		timers[n].timerStatus=UNUSED;
		timers[n].cyclic=false;
		timers[n].preset=0;
		timers[n].elapsedTicksSinceStart=0;
		timers[n].rbuf=0;
		timers[n].evTimeout=0;
	}
}


/**
 * This function has to be called from the timer irq to
 * cyclically check if a timeout has happened.
 */
bool tick(void){
  bool timeoutAvailable=false;
  
  // Iterate over the timers and enque
  // timer events once a timeout has happend.
  uint8_t i;
  for(i=0; i<MAX_TIMERS; i++){
  
    if(timers[i].timerStatus==ON){
      
      timers[i].elapsedTicksSinceStart++;
                  
      if(timers[i].elapsedTicksSinceStart==timers[i].preset){

    	if(timers[i].cyclic==true){
    		timers[i].elapsedTicksSinceStart = 0;
    	}else{
            // timeout time elapsed.
            timers[i].timerStatus=OFF; // stop timer
    	}

    	__disable_interrupt();
        fifoPut(timers[i].rbuf, timers[i].evTimeout); // enqueue timeout event
    	__enable_interrupt();
        timeoutAvailable=true;
      }
    }
  }
  
  return timeoutAvailable;
}

//
// Functions for all timers
//

uint8_t timerCreate(FIFO_T *rbuf, uint16_t eventId, bool cyclic){
  uint8_t n=0; 
  
  while(n<MAX_TIMERS){
    if(timers[n].timerStatus==UNUSED){
    	timers[n].cyclic=cyclic;
        timers[n].timerStatus = OFF;
        timers[n].rbuf = rbuf;
        timers[n].evTimeout=eventId;
        return n;
    }
    n++;
  }
  return NO_TIMERS_LEFT; // no timers left
}

void timerErase(uint8_t t_id){
  // not jet impl.
}

//
// Timer functions for a single timer
//

void timerPause(uint8_t t_id){
  if(t_id>(MAX_TIMERS-1)) return;
  timers[t_id].timerStatus=PAUSE;
}

void timerCont(uint8_t t_id){
  if(t_id>(MAX_TIMERS-1)) return;
  timers[t_id].timerStatus=ON;
}

void timerClear(uint8_t t_id){
  if(t_id>(MAX_TIMERS-1)) return;
  timers[t_id].preset=0;
}

TIMER_STATE_T timerStart(uint8_t t_id, uint8_t ticks){
  if(t_id>MAX_TIMERS) return UNKNOWN;

  timers[t_id].preset=ticks;
  timers[t_id].elapsedTicksSinceStart = 0;
  timers[t_id].timerStatus = ON;
  return ON;
}

void timerStop(uint8_t t_id){
  if(t_id>(MAX_TIMERS-1)) return;

  timers[t_id].timerStatus=OFF;
}

TIMER_STATE_T timerStatus(uint8_t t_id){
	if(t_id>(MAX_TIMERS-1)) return UNKNOWN;

	return timers[t_id].timerStatus;
}

